"""
Модуль класса представления для работы с данными о пользователях.
"""
from flask import Blueprint
import requests
from flask.views import MethodView
from src.utils.response import json_response
from src.models import StatsModel


acquisition_bp = Blueprint('stats_acquisition', __name__)


class StatsAcquisitionView(MethodView):
    """
    Класс представления для работы с общими данными о пользователях.
    """
    def __init__(self):
        self.model = StatsModel()

    def get(self, info_type):
        """
        Метод получения общих данных.
        :param type: тип запрашиваемых данных
        :return: полученные от сервера данные, обернутые в статус-код
        """
        try:
            if info_type == 1:
                data = self.model.get_number_of_users()
            elif info_type == 2:
                data = self.model.get_number_of_messages_all_users()
            elif info_type == 3:
                data = dict(self.model.get_number_messages_of_each_user())
            elif info_type == 4:
                data = dict(self.model.get_character_count_of_each_user())
        except requests.exceptions.RequestException:
            return json_response.not_found()
        return json_response.success(data)


class StatAcquisitionView(MethodView):
    """
    Класс представления для получения данных о конкретном пользователе.
    """
    def __init__(self):
        self.model = StatsModel()

    def get(self, username, info_type):
        """
        Метод получения статистики пользователя.
        :param username: имя пользователя
        :param type: тип запрашиваемых данных
        :return: полученные от сервера данные, обернутые в статус-код
        """
        try:
            if info_type == 1:
                data = self.model.get_number_of_messages_one_user(username)
            elif info_type == 2:
                data = self.model.get_number_of_character_one_user(username)
        except requests.exceptions.RequestException:
            return json_response.not_found()
        return json_response.success(data)


class UsersListAcquisitionView(MethodView):
    """
    Класс представления для получения списка пользователей.
    """
    def __init__(self):
        self.model = StatsModel()

    def get(self):
        """
        Метод получения списка пользователей.
        :return: полученные от сервера данные, обернутые в статус-код
        """
        data = {}
        try:
            data = self.model.get_users_list()
        except requests.exceptions.RequestException:
            return json_response.not_found(data)
        return json_response.success(data)

acquisition_bp.add_url_rule('/<int:info_type>/',
                            view_func=StatsAcquisitionView.as_view('stats_acquisition_view'))
acquisition_bp.add_url_rule('/<username>/<int:info_type>/',
                            view_func=StatAcquisitionView.as_view('stat_acquisition_view'))
acquisition_bp.add_url_rule('', view_func=UsersListAcquisitionView.
                            as_view('users_list_acquisition_view'))



"""
Модуль старта приложения.
"""
from flask import Flask
from views.stats_acquisition import acquisition_bp


def create_app():
    """
    Метод создания Flask приложения.
    :return: приложение Flask
    """
    app = Flask(__name__)  # Создание приложения Flask.
    app.debug=True
    app.config.from_object('config.Config')  # Проброс конфигурационных параметров.
    app.register_blueprint(acquisition_bp, url_prefix='/acquisition')

    return app



import sqlalchemy as db
from sqlalchemy import func


class StatsModel:
    """
    Класс для работы с БД.
    """
    def __init__(self):
        self.engine = db.create_engine('sqlite:///scrapy_forum.db')
        self.connection = self.engine.connect()
        self.metadata = db.MetaData()
        self.forum = db.Table('forum', self.metadata, autoload=True, autoload_with=self.engine)

    # print(forum.columns.keys()) # заголовки столбцов

    def get_number_of_messages_one_user(self, username):
        """
        Метод для получения количества сообщений конкретного пользователя.
        :param username: имя пользователя
        :return: ответ сервера
        """
        query = db.select([func.count()]).where(self.forum.columns.username == username)
        result = self.connection.execute(query).fetchone()
        return result[0]

    def get_number_of_character_one_user(self, username):
        """
        Метод для получения количества символов во всех сообщениях пользователя.
        :param username: имя пользователя
        :return: ответ сервера
        """
        query = db.select([func.length(self.forum.columns.user_message)]).\
            where(self.forum.columns.username == username)
        result = self.connection.execute(query).fetchall()
        number_of_characters = 0
        for number in result:
            number_of_characters += number[0]
        return number_of_characters

    def get_number_of_users(self):
        """
        Метод для получения количества пользователей.
        :return: ответ сервера
        """
        query = db.select([self.forum.columns.username]).group_by(self.forum.columns.username)
        result = self.connection.execute(query).fetchall()
        # print(result) список пользователей
        return len(result)

    def get_number_of_messages_all_users(self):
        """
        Метод для получения количества сообщений всех пользователей.
        :return: ответ сервера
        """
        query = db.select([func.count(self.forum.columns.username)])
        result = self.connection.execute(query).fetchone()
        return result[0]

    def get_number_messages_of_each_user(self):
        """
        Метод для получения количества сообщений каждого пользователя.
        :return: ответ сервера
        """
        query = db.select([self.forum.columns.username, func.count()]).\
            group_by(self.forum.columns.username)
        result = self.connection.execute(query).fetchall()
        return result

    def get_character_count_of_each_user(self):
        """
        Метод для получения количества символов по сообщениям пользователя для всех пользователей.
        :return: ответ сервера
        """
        cte_query = db.select([self.forum.columns.username.label('username'),
                               func.length(self.forum.columns.user_message).
                              label('message_length')]).\
            group_by(self.forum.columns.username, self.forum.columns.user_message)
        cte_query = cte_query.cte('all_messages')
        query2 = db.select(cte_query.c.username, func.sum(cte_query.c.message_length)).\
            group_by(cte_query.c.username)
        result = self.connection.execute(query2).fetchall()
        return result

    def get_users_list(self):
        """
        Метод для получения списка пользователей.
        :return: ответ сервера
        """
        query = query = db.select([func.distinct(self.forum.columns.username)])
        result = self.connection.execute(query).fetchall()
        data = []
        for element in result:
            data.append(element[0])
        return data



from flask import jsonify


class JsonResponse:
    """
    Класс обертывания ответа сервера в коды состояния.
    """
    HTTP_CODE_OK = 200
    HTTP_CODE_NOT_FOUND = 404

    def success(self, data: dict = None):
        """
        Метод обертывания ответа сервера в статус-код 200.

        :param data: ответ сервера
        :type data: dict
        :return: ответ сервера
        """
        return self._response(self.HTTP_CODE_OK, data)

    def not_found(self, data: dict = None):
        """
        Метод обертывания ответа сервера в статус-код 404.

        :param data: ответ сервера
        :type data: dict
        :return: ответ сервера
        """
        return self._response(self.HTTP_CODE_NOT_FOUND, data)

    @classmethod
    def _response(cls, code: int, data: dict = None):
        """
        Метод оборачивания ответа сервера в json.

        :param code: статус-код
        :param data: ответ сервера
        :return: обёрнутый ответ сервера
        """
        if data is not None:
            return jsonify(data), code
        return '', code


json_response = JsonResponse()



import os
from setuptools import setup


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(name='statsBotServer',
      version='0.1',
      description='Server of the bot for collecting statistics',
      url='https://github.com/MinaevaKsenia/statsBot',
      author='Minaeva Ksenia',
      author_email='minaeva.ksen@gmail.com',
      zip_safe=False,
      packages=['src', 'src/utils', 'src/views'],
      long_description=read('README.md'),
      include_package_data=True,
      test_suite="tests"
      )



import os


class Config:
    """ Класс с конфигурацией для FLASK приложения. """
    FLASK_ENV = os.getenv('FLASK_ENV', 'production')
    DEBUG = os.getenv('DEBUG', True)
    DB_CONNECTION = os.getenv('DB_CONNECTION', 'db.sqlite')
    SECRET_KEY = os.getenv('SECRET_KEY', '4f4g5sdew98vsef1t2a5v7b').encode()
